
#include <stdio.h>
#include <math.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <sys/time.h>
#include "input.h"
#include "xoutput.h"
#include "leetft.h"

static double intensity_to_voltage_srgb( double val )
{
    /* Handle invalid values before doing a gamma transform. */
    if( val < 0.0 ) return 0.0;
    if( val > 1.0 ) return 1.0;

    /* sRGB uses an exponent of 2.4 in its nonstandard curve, but it
     *      * still advertises a display gamma of 2.2.
     *           */
    if( val <= 0.0031308 ) {
        return 12.92 * val;
    }
    return ( ( 1.055 * pow( val, 1.0 / 2.4 ) ) - 0.055 );
}

int main( int argc, char **argv )
{
    output_api_t *output = xoutput_get_output();
    input_t *in = input_new();
    int width, height, stride;
    ft_font_t *font;
    ft_string_t *fts;
    double pixel_aspect;
    int i, j;
    int pos = 80*65536;
    int accel = 70000;
    // int accel = 4096;
    int paused = 1;
    int size = 12;

    width = 640;
    height = 120;
    output->init( width, height );
    stride = output->get_output_stride();

    if( argc > 1 ) {
        size = atoi( argv[ 1 ] );
    }
    fprintf( stderr, "using font size %d\n", size );

    pixel_aspect = 1.0;
    font = ft_font_new( "FreeSans.ttf", size, pixel_aspect );
    if( !font ) {
        fprintf( stderr, "can't open font?\n" );
        return 1;
    }

    fts = ft_string_new( font );

    for(;;) {
        output->poll_events( in );
        if( input_quit_requested( in ) ) break;
        if( input_pause_requested( in ) ) paused = !paused;
        input_next_frame( in );

        ft_string_set_text( fts, "Top is gamma correct, bottom not correct", pos );
        if( !paused ) {
            pos += accel;
            if( pos < 0 ) { pos = 0; accel = -accel; }
            if( pos > 65536*400 ) { accel = -accel; }
        }

        output->lock_output_buffer();
        memset( output->get_output_buffer(), 0, stride * height );

        for( i = 0; i < ft_string_get_height( fts ); i++ ) {
            unsigned char *curscanline = output->get_output_buffer() + (stride * 30) + (stride*i);
            unsigned char *fdata = ft_string_get_buffer( fts ) + (i * ft_string_get_stride( fts ));
            for( j = 0; j < ft_string_get_width( fts ); j++ ) {
                if( fdata[ j ] ) {
                    (curscanline + (j*4))[ 0 ] = (int)((255.0 * intensity_to_voltage_srgb( ((double) fdata[ j ]) / 255.0 ))+0.5);
                    // (curscanline + (j*4))[ 0 ] = fdata[ j ];
                    (curscanline + (j*4))[ 1 ] = (curscanline + (j*4))[ 0 ];
                    (curscanline + (j*4))[ 2 ] = (curscanline + (j*4))[ 0 ];
                    (curscanline + (j*4))[ 3 ] = 0xff;
                }
            }
        }

        for( i = 0; i < ft_string_get_height( fts ); i++ ) {
            unsigned char *curscanline = output->get_output_buffer() + (stride * 60) + (stride*i);
            unsigned char *fdata = ft_string_get_buffer( fts ) + (i * ft_string_get_stride( fts ));
            for( j = 0; j < ft_string_get_width( fts ); j++ ) {
                if( fdata[ j ] ) {
                    // (curscanline + (j*4))[ 0 ] = (int)((255.0 * intensity_to_voltage_srgb( ((double) fdata[ j ]) / 255.0 ))+0.5);
                    (curscanline + (j*4))[ 0 ] = fdata[ j ];
                    (curscanline + (j*4))[ 1 ] = (curscanline + (j*4))[ 0 ];
                    (curscanline + (j*4))[ 2 ] = (curscanline + (j*4))[ 0 ];
                    (curscanline + (j*4))[ 3 ] = 0xff;
                }
            }
        }
        output->unlock_output_buffer();
        output->show_frame();
        usleep( 20 );
    }

    output->shutdown();
    return 0;
}

